@@ -1,24 +1,34 @@
-# $Rev: 74 $
-# $Id: Glue.pm 74 2003-07-23 00:05:07Z afoxson $
-
+# $Revision: 1.3 $
+# $Id: Glue.pm,v 1.3 2006/07/04 07:19:51 afoxson Exp $
+#
 # Bot::CPAN::Glue - Deep magic for Bot::CPAN
 # Copyright (c) 2003 Adam J. Foxson. All rights reserved.
 
-# This program is free software; you can redistribute it and/or modify
-# it under the same terms as Perl itself.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of the GNU General Public License as published by the Free Software
+# Foundation; either version 2 of the License, or (at your option) any later
+# version.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
 
 package Bot::CPAN::Glue;
 
-require 5.006;
+require 5.008;
 
 use strict;
+use warnings;
 use POE;
 use vars qw(@ISA @EXPORT $VERSION %commands);
 use Bot::CPAN::BasicBot;
+use Error qw(:try);
+use Bot::CPAN::E::NoDist;
+use Bot::CPAN::E::NoAuth;
+use Bot::CPAN::E::NoMod;
+use Bot::CPAN::E::Unknown;
+use Class::Phrasebook;
 use constant NOT_A_COMMAND   => 0;
 use constant PUBLIC_COMMAND  => 1<<0;
 use constant PRIVATE_COMMAND => 1<<1;
@@ -42,10 +52,8 @@ use Attribute::Handlers autotie => {
 	'__CALLER__::Admin'   => __PACKAGE__,
 };
 
-($VERSION) = sprintf "%.02f", (('$Rev: 74 $' =~ /\s+(\d+)\s+/)[0] / 100);
-@ISA       = qw(Bot::CPAN::BasicBot);
-
-local $^W;
+($VERSION) = '$Revision: 1.3 $' =~ /\s+(\d+\.\d+)\s+/;
+@ISA = qw(Bot::CPAN::BasicBot);
 
 # this is the command handler, here we are ultimately responsible for
 # accepting or rejecting a command
@@ -63,7 +71,32 @@ sub _command {
 	$message->{data} = $command;
 	$self->log("DEBUG: _command: $command, $module_or_author\n") if
 		$self->debug;
-	$self->_dispatch($message, $module_or_author);
+    try {
+	    $self->_dispatch($message, $module_or_author);
+    }
+    catch Bot::CPAN::E::NoDist with {
+        my $E = shift;
+        $self->_print($message, $self->phrase('NO_DISTRIBUTION', {distribution => $E->{'-text'}}));
+    }
+    catch Bot::CPAN::E::NoMod with {
+        my $E = shift;
+        $self->_print($message, $self->phrase('NO_MODULE', {module => $E->{'-text'}}));
+    }
+    catch Bot::CPAN::E::NoAuth with {
+        my $E = shift;
+        $self->_print($message, $self->phrase('NO_AUTHOR', {author => $E->{'-text'}}));
+    }
+    catch Bot::CPAN::E::Unknown with {
+        $self->_print($message, $self->phrase('UNKNOWN'));
+    }
+    catch Error::Simple with {
+        my $E = shift;
+        my $error = $E->{'-text'};
+        my $file = $E->{'-file'};
+        my $line = $E->{'-line'};
+        $self->log("DEBUG: COMMAND FAILURE: $error at $file line $line.\n");
+        $self->_print($message, $self->phrase('COMMAND_FAILURE'));
+    };
 }
 
 sub _commands {
@@ -86,6 +119,7 @@ sub _dispatch {
 				who => $message->{who},
 				channel => $message->{channel},
 				arguments => [$message, $module_or_author],
+                address => 1,
 				data => $message->{data},
 		});
 	} 
@@ -101,16 +135,16 @@ sub _fork_handler {
 	my ($self, $body, $wheel_id) = @_[0, ARG0, ARG1];
 	chomp $body;
 
-	# this is not particularly endearing, but it has to be done *sigh*
-	# why? if we don't do this here than the requesting user will also be
-	# sent internal CPANPLUS debugging info. this ensures that the user gets
-	# sent only what they ask for
-	my $passthrough_pattern = __PACKAGE__;
-	unless ($body =~ /^$passthrough_pattern:\s/) {
-		$self->log("$body\n");
-		return;
-	}
-	$body =~ s/$passthrough_pattern: //;
+    # this is not particularly endearing, but it has to be done *sigh*
+    # why? if we don't do this here than the requesting user will also be
+    # sent internal package info. this ensures that the user gets
+    # sent only what they ask for
+    my $passthrough_pattern = __PACKAGE__;
+    unless ($body =~ /^$passthrough_pattern:\s/) {
+        $self->log("$body\n");
+        return;
+    }
+    $body =~ s/$passthrough_pattern: //;
 
 	my $args = $self->{forks}->{$wheel_id}->{args};
 
@@ -174,10 +208,12 @@ sub new {
 	while (my ($key, $value) = splice @_, 0, 2) {
 		if ($key eq 'news_server' ||
 			$key eq 'group' ||
+			$key eq 'nickserv_password' ||
 			$key eq 'adminhost' ||
 			$key eq 'reload_indices_interval' ||
 			$key eq 'policy' ||
 			$key eq 'search_max_results' ||
+			$key eq 'inform_channel_of_new_ratings' ||
 			$key eq 'inform_channel_of_new_uploads') {
 				push @my_args, $key, $value;
 		}
@@ -190,11 +226,13 @@ sub new {
 
 	# set up some sane defaults
 	$upstream->set('news_server', 'nntp.perl.org');
+	$upstream->set('nickserv_password', undef);
 	$upstream->set('adminhost', qr/\b\B/); # default impossible match, Fletch++
 	$upstream->set('search_max_results', 20);
 	$upstream->set('group', 'perl.cpan.testers');
 	$upstream->set('reload_indices_interval', 300);
 	$upstream->set('inform_channel_of_new_uploads', 60);
+	$upstream->set('inform_channel_of_new_ratings', 60);
 	$upstream->set('policy', {});
 
 	while (my ($key, $value) = splice @my_args, 0, 2) {
@@ -203,6 +241,17 @@ sub new {
 
 	$upstream->_verify_policy();
 
+	# mix in the phrasebook
+	my $pm = $INC{'Bot/CPAN.pm'};
+	$pm =~ s/\.pm//;
+	my $pb = Class::Phrasebook->new(undef, "$pm/phrases.xml");
+
+	die "Unable to load phrasebook.\n" unless defined $pb;
+
+	$pb->remove_new_lines(1);
+	$pb->load("EN");
+	$upstream->set('pb', $pb);
+
 	return $upstream;
 }
 
@@ -300,6 +349,8 @@ sub said {
 	return undef unless $message->{address}; 
 
 	$self->_command($message);
+
+    return;
 }
 
 sub _verify_policy {
@@ -394,7 +445,7 @@ sub _verify_auth {
 			return 1;
 		}
 		else {
-			$self->log("$message->{who} tried to use admin command $command\n");
+			$self->log("$message->{userhost} tried to use admin command $command\n");
 			$message->{body} = "'$command' is an admin only command";
 			$self->_return($message);
 			return;
@@ -404,6 +455,59 @@ sub _verify_auth {
 	return 1;
 }
 
+sub phrase {
+	my $self = shift;
+	my ($filename, $line, $caller) = (caller(1))[1..3];
+	my $actual = shift if not $caller;
+	my ($phrase, $subphrase);
+	my $pb = $self->get('pb');
+
+	if (@_) {
+		if (ref $_[0] and ref $_[0] eq 'HASH') {
+			$caller = $actual if $actual;
+			unless ($phrase = $pb->get($caller, @_)) {
+				$phrase = __PACKAGE__ . ": phrase not found for " .
+					"$caller with hash arg at $filename:$line";
+			}
+		}
+		else {
+			$subphrase = shift;
+
+			if ($subphrase eq uc($subphrase)) {
+				$subphrase = $actual if $actual;
+				unless ($phrase = $pb->get($subphrase, @_)) {
+					$phrase = __PACKAGE__ . ": phrase not found for " .
+						"global $subphrase at $filename:$line";
+				}
+			}
+			else {
+				if ($actual) {
+					$caller = $actual;
+				}
+				else {
+					$caller .= "($subphrase)";
+				}
+				unless ($phrase = $pb->get($caller, @_)) {
+					$phrase = __PACKAGE__ . ": phrase not found for " .
+						"$caller($subphrase) with args at $filename:$line";
+				}
+			}
+		}
+	}
+	else {
+		$caller = $actual if $actual;
+		unless ($phrase = $pb->get($caller)) {
+			$phrase = __PACKAGE__ . ": phrase not found for " .
+				"$caller without args at $filename:$line";
+		}
+	}
+
+	$phrase =~ s/^\s+//;
+	$phrase =~ s/\s+$//;
+
+	return $phrase;
+}
+
 # and here are the attribute handlers
 
 sub Fork : ATTR(CODE)    { $commands{*{$_[1]}{NAME}}[PERM] |= FORK }
